/*
 * CS559 Maze Project
 *
 * Class file for Edge class. Stores data about edges in the maze.
 *
 * (c) Stephen Chenney, University of Wisconsin at Madison, 2001-2002
 *
 */

#include <stdio.h>
#include "Edge.h"
#include "Cell.h"


const char  Edge::LEFT = 0;
const char  Edge::RIGHT = 1;
const char  Edge::ON = 2;
const char  Edge::NEITHER = 3;

const char  Edge::START = 0;
const char  Edge::END = 1;


Edge::Edge(int i, Vertex *start, Vertex *end, float r, float g, float b)
{
    index = i;
    endpoints[START] = start;
    endpoints[END] = end;

    neighbors[0] = neighbors[1] = NULL;

    opaque = true;

    color[0] = r;
    color[1] = g;
    color[2] = b;
}


char
Edge::Cell_Side(Cell *cell)
{
    if ( cell == neighbors[LEFT] )
	return LEFT;
    else if ( cell == neighbors[RIGHT] )
	return RIGHT;
    
    return NEITHER;
}


char
Edge::Point_Side(float x, float y)
{
    // Compute the determinant: | xs ys 1 |
    //                          | xe ye 1 |
    //                          | x  y  1 |
    // Use its sign to get the answer.

    float   det;

    det = endpoints[START]->posn[Vertex::X]
          * ( endpoints[END]->posn[Vertex::Y] - y )
        - endpoints[START]->posn[Vertex::Y]
          * ( endpoints[END]->posn[Vertex::X] - x )
	+ endpoints[END]->posn[Vertex::X] * y
	- endpoints[END]->posn[Vertex::Y] * x;
    
    if ( det == 0.0 )
	return ON;
    else if ( det > 0.0 )
	return LEFT;
    else
	return RIGHT;
}


